/*++

Copyright (c) 2009	Microsoft Corporation

Module Name:
	Device.h

Abstract:
	Definitions of PCI Driver Routine Functions

Revision History:
	Who 		When		What						
	--------	--------	-----------------------------------------------
	Chen Tian	16-03-09	Wrtitten For NetFPGA2 Card in Microsoft Research Asia
Notes:

--*/

#ifndef _PCIE_DEVICE_H_
#define _PCIE_DEVICE_H_

#include "nf2kernel.h"

//
// The device extension for the device object
//
typedef struct _DEVICE_EXTENSION {
	WDFDEVICE			Device;
	WDFINTERRUPT		Interrupt;
	ULONG				Status;		// the irp status register
	ULONG				Mask;		// the irp mask register

	 // For handling IOCTLs - required if the upper edge is NDIS
	WDFQUEUE			IOCtrlQueue;
	WDFQUEUE			PendingIoctlQueue;
	WDFQUEUE			WriteQueue;
	WDFQUEUE			PendingWriteQueue;
	WDFQUEUE			PendingReadQueue;

	WDFDMAENABLER		DMAEnabler;

	// These fields are added for NetFPGA
	/* Address in memory of board */
	NDIS_PHYSICAL_ADDRESS	MemPhysAddress;
	ULONG					MemPhysRange;
	PVOID					MemVirAddress;

	// Seperate driver or not
	ULONG					IsUpperEdgeNdis;

	// Common buffer for DMA transfer
	WDFCOMMONBUFFER		WdfSendCommonBuffer;
	PVOID					HwSendMemAllocVa;
	NDIS_PHYSICAL_ADDRESS	HwSendMemAllocPa;
	ULONG					SendinProgress;
	ULONG					RecvinProgress;

	WDFCOMMONBUFFER		WdfReceiveCommonBuffer;
	PVOID					HwRcvMemAllocVa;
	NDIS_PHYSICAL_ADDRESS	HwRcvMemAllocPa;

	/* Control card */
	char	is_ctrl;

	// MAC Address of the device
	UCHAR					PermanentAddress[ETH_LENGTH_OF_ADDRESS];
	UCHAR					CurrentAddress[ETH_LENGTH_OF_ADDRESS];

	/* Transmit Buffers */
	struct txbuff				*txbuff;

	WDFDMATRANSACTION		currentDmaTransaction;
	WDFREQUEST 			currentRequest;

	/* Current and available txbuff */
	int	wr_txbuff;
	int	rd_txbuff;
	int	free_txbuffs;
	int	free_txbuffs_port[MAX_IFACE];

	/* Packet pool for incomming packets */
	struct _NF2PACKET	*ppool;


	/* What number interface is this? */
	USHORT	Iface;

	/* The current buffer to process and the last buffer used from the pool */
	struct nf2_packet *rd_pool, *wr_pool;

}  DEVICE_EXTENSION, *PDEVICE_EXTENSION;

//
// This will generate the function named PCIEGetDeviceContext to be use for
// retreiving the DEVICE_EXTENSION pointer.
//
WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(DEVICE_EXTENSION, PCIEGetDeviceContext)

#pragma warning(disable:4127) // avoid conditional expression is constant error with W4



NTSTATUS
PCIEEvtDeviceAdd(
	IN WDFDRIVER			Driver,
	IN PWDFDEVICE_INIT	DeviceInit
	);

NTSTATUS
PCIEInitializeDeviceExtension(
	IN PDEVICE_EXTENSION pDevExt
	);

NTSTATUS
PCIESetIdleAndWakeSettings(
	IN PDEVICE_EXTENSION FdoData
	);

NTSTATUS
PCIEEvtDevicePrepareHardware (
	WDFDEVICE			Device,
	WDFCMRESLIST		Resources,
	WDFCMRESLIST		ResourcesTranslated
	);

NTSTATUS
PCIEEvtDeviceReleaseHardware(
	IN	WDFDEVICE		Device,
	IN	WDFCMRESLIST	ResourcesTranslated
	);

NTSTATUS
PCIEEvtDeviceD0Entry(
	IN	WDFDEVICE					Device,
	IN	WDF_POWER_DEVICE_STATE	PreviousState
	);

NTSTATUS
PCIEEvtDeviceD0Exit(
	IN	WDFDEVICE					Device,
	IN	WDF_POWER_DEVICE_STATE	TargetState
	);

VOID
PCIEDeviceShutdown(
	IN PDEVICE_EXTENSION pDevExt
	);

VOID
PCIEDeviceReset(
	IN PDEVICE_EXTENSION pDevExt
	);

VOID 
PCIEDeviceWait(
	INT ms
	);

BOOLEAN
PciDrvReadRegistryKeyValue(
	IN	PWDFDEVICE_INIT	DeviceInit,
	IN	PWCHAR			Name,
	OUT PULONG			Value
	);

void
ReadAdapterInfo (
	IN PDEVICE_EXTENSION pDevExt
	);

VOID
PciDrvEvtIoWrite(
	IN WDFQUEUE		Queue,
	IN WDFREQUEST	Request,
	IN size_t			Length
	);

BOOLEAN
PCIEEvtProgramWriteDma(
	IN	WDFDMATRANSACTION		Transaction,
	IN	WDFDEVICE				Device,
	IN	PVOID					Context,
	IN	WDF_DMA_DIRECTION		Direction,
	IN	PSCATTER_GATHER_LIST	SgList
	);

VOID
PCIEWriteRequestComplete(
	IN WDFDMATRANSACTION	DmaTransaction,
	IN NTSTATUS				Status
	);

VOID
PciDrvEvtIoRead(
	IN WDFQUEUE		Queue,
	IN WDFREQUEST	Request,
	IN size_t			Length
	);

BOOLEAN
PCIEEvtProgramReadDma(
	IN	WDFDMATRANSACTION		Transaction,
	IN	WDFDEVICE				Device,
	IN	WDFCONTEXT				Context,
	IN	WDF_DMA_DIRECTION		Direction,
	IN	PSCATTER_GATHER_LIST	SgList
	);

/*
 * Disable interrupts
 */
void 
NF2ResetAll (
	IN PDEVICE_EXTENSION pDevExt
	);

void 
NF2ResetCPCI (
	IN PDEVICE_EXTENSION pDevExt
	);

NTSTATUS
NF2InterruptEnable(
    IN WDFINTERRUPT  Interrupt,
    IN WDFDEVICE     AssociatedDevice
    );

NTSTATUS
NF2InterruptDisable(
    IN WDFINTERRUPT  Interrupt,
    IN WDFDEVICE     AssociatedDevice
    );

ULONG
ReadRegisterUlong (
    IN PDEVICE_EXTENSION pDevExt,
    IN ULONG address
    );

VOID
WriteRegisterUlong(
    IN PDEVICE_EXTENSION pDevExt,
    IN ULONG address,
    IN ULONG value
    ); 

#define ASSOC_WRITE_REQUEST_WITH_DMA_TRANSACTION 1

#endif
