/*++

Copyright (c) 2009	Microsoft Corporation

Module Name:
	Nf2kernel.h

Abstract:
	Header file for kernel driver, Revised From NetFPGA 2 Linux Kernel Driver

Revision History:
	Who 		When		What						
	--------	--------	-----------------------------------------------
	Chen Tian	16-03-09	Wrtitten For NetFPGA2 Card in Microsoft Research Asia
Notes:

--*/

#ifndef _NF2KERNEL_H
#define _NF2KERNEL_H

#include "Device.h"

/* Define PCI Vendor and device IDs for the NF2 card */
#define PCI_VENDOR_ID_STANFORD		0xFEED
#define PCI_DEVICE_ID_STANFORD_NF2	0x0001

/* How many interfaces does a single card support */
#ifndef MAX_IFACE
#define MAX_IFACE	4
#endif

/* Transmit timeout period */
#define NF2_TIMEOUT	(2 * HZ)

/* How many transmit buffers to allocate */
#define NUM_TX_BUFFS	16

/* How many transmit buffers to allocate */
#define NUM_RX_BUFFS	8

/* How large is the largest DMA transfer */
#define MAX_DMA_LEN	2048

/* Major device number for user devices */
#define NF2_MAJOR 0   /* dynamic major by default */

/* Maximum transmission size -- this should be 
 * max packet size - 14 for the Ethernet header */
#define MTU 1986

#define ETH_LENGTH_OF_ADDRESS 6

/*
 * Buffer to hold packets to be transmitted
 */
typedef struct _TXBUFF {
	struct sk_buff *skb;
	char * buff;
	USHORT len;
	USHORT iface;
} TXBuff;


/*
 * A structure representing an in-flight packet being received.
 */
typedef struct _NF2PACKET {
	struct NF2Packet *next;
	struct _NF2INTERFACE *dev;
	int len;
	unsigned int data[MAX_DMA_LEN + 2];
} NF2Packet;

/*
 * Functions
 */
UCHAR 
NF2IsControlBoard ( IN PVOID virAddress);

NTSTATUS
SetupControlBoard ( IN struct _DEVICE_EXTENSION * pDevExt);

/*
 * Variables
 */
extern int timeout;
extern int rx_pool_size;
extern int tx_pool_size;
extern int nf2_major;
extern int nf2_minor;

#endif	/* _NF2KERNEL_H */
