/*++

Copyright (c) 2009	Microsoft Corporation

Module Name:
	NDISWDM.C

Abstract:
	This module is revised from NDISEDGE of WINDDK examples,
	contains structure definitons and function prototypes.


Revision History:
	Who 		When		What						
	--------	--------	-----------------------------------------------
	Chen Tian	16-03-09	Wrtitten For NetFPGA2 Card in Microsoft Research Asia
Notes:

--*/

#ifndef _NDISWDM_H
#define _NDISWDM_H

#pragma warning(disable:4201)  // nameless struct/union warning
#pragma warning(disable:4214)  // bit field types other than int warning

#define WIN9X_COMPAT_SPINLOCK

#include <ndis.h>

#pragma warning(default:4201)
#pragma warning(default:4214)

#include <wdf.h>
#include <WdfMiniport.h>

#include "..\inc\NuIoUser.h"
#include "..\inc\Public.h"
//
// To use strsafe function on 9x, ME, and Win2K Oses, we
// have to define NTSTRSAFE_LIB before including this header file and explicitly
// link to ntstrsafe.lib. If your driver is just target for XP and above, there is
// no need to define NTSTRSAFE_LIB and link to the lib.
//
#define NTSTRSAFE_LIB
#include <ntstrsafe.h>
#include <ntintsafe.h>

#if defined(NDIS50_MINIPORT)
    #define MP_NDIS_MAJOR_VERSION       5
    #define MP_NDIS_MINOR_VERSION       0
#elif defined(NDIS51_MINIPORT)
    #define MP_NDIS_MAJOR_VERSION       5
    #define MP_NDIS_MINOR_VERSION       1
#else
#error Unsupported NDIS version
#endif

#if defined(NDIS50_MINIPORT)
EXPORT
UINT
NdisGetVersion(
    VOID
    );
#endif

//-----------------------------------------------------------------------------
// 4127 -- Conditional Expression is Constant warning
//-----------------------------------------------------------------------------
#define WHILE(constant) \
__pragma(warning(disable: 4127)) while(constant); __pragma(warning(default: 4127))


#define NIC_TAG                             ((ULONG)'MDWN')

// media type, we use ethernet, change if necessary
#define NIC_MEDIA_TYPE                    NdisMedium802_3

// we use Internal, change to Pci, Isa, etc. properly
#define NIC_INTERFACE_TYPE                NdisInterfaceInternal

//
// Update the driver version number every time you release a new driver
// The high byte is the major version. The low byte is the minor version.
// Also make sure that VER_FILEVERSION specified in the .RC file also
// matches with the driver version because NDISTESTER checks for that.
//
#define NIC_VENDOR_DRIVER_VERSION       0x00010000

// change to your company name instead of using Microsoft
#define NIC_VENDOR_DESC                 "Stanford"

// Highest byte is the NIC byte plus three vendor bytes, they are normally
// obtained from the NIC
#define NIC_VENDOR_ID                    0x00FFFFFF

#define PROTOCOL_INTERFACE_NAME L"\\??\\NdisProt"

typedef UCHAR   NIC_MAC_ADDRESS[6];

#define     ETH_HEADER_SIZE             14
#define     ETH_MAX_DATA_SIZE           1500
#define     ETH_MAX_PACKET_SIZE         ETH_HEADER_SIZE + ETH_MAX_DATA_SIZE
#define     ETH_MIN_PACKET_SIZE         60


#define ETH_IS_LOCALLY_ADMINISTERED(Address) \
                (BOOLEAN)(((PUCHAR)(Address))[0] & ((UCHAR)0x02))

#define NIC_MAX_MCAST_LIST              32
#define NIC_MAX_BUSY_SENDS              512
//#define NIC_MAX_BUSY_SENDS              32
#define NIC_MAX_SEND_PKTS               5
#define NIC_MAX_BUSY_RECVS              512
#define NIC_MAX_LOOKAHEAD               ETH_MAX_DATA_SIZE
#define NIC_BUFFER_SIZE                 ETH_MAX_PACKET_SIZE
#define NIC_LINK_SPEED                  10000000    // in 100 bps
#define NIC_SEND_LOW_WATERMARK         NIC_MAX_BUSY_SENDS/4
#define NIC_ADAPTER_NAME_SIZE          128

//
// Buffer size passed in NdisMQueryAdapterResources
// We should only need three adapter resources (IO, interrupt and memory),
// Some devices get extra resources, so have room for 10 resources
//
#define NIC_RESOURCE_BUF_SIZE           (sizeof(NDIS_RESOURCE_LIST) + \
                                        (10*sizeof(CM_PARTIAL_RESOURCE_DESCRIPTOR)))

#define NIC_SUPPORTED_FILTERS ( \
                NDIS_PACKET_TYPE_DIRECTED   | \
                NDIS_PACKET_TYPE_MULTICAST  | \
                NDIS_PACKET_TYPE_BROADCAST  | \
                NDIS_PACKET_TYPE_PROMISCUOUS | \
                NDIS_PACKET_TYPE_ALL_MULTICAST)

#define fMP_RESET_IN_PROGRESS               0x00000001
#define fMP_DISCONNECTED                    0x00000002
#define fMP_HALT_IN_PROGRESS                0x00000004
#define fMP_SURPRISE_REMOVED                0x00000008
#define fMP_RECV_LOOKASIDE                  0x00000010
#define fMP_INIT_IN_PROGRESS                0x00000020
#define fMP_SEND_SIDE_RESOURCE_ALLOCATED    0x00000040
#define fMP_RECV_SIDE_RESOURCE_ALLOCATED    0x00000080
#define fMP_POST_READS                      0x00000100
#define fMP_POST_WRITES                     0x00000200

//
// Message verbosity: lower values indicate higher urgency
//
/*#define MP_VERY_LOUD    5
#define MP_LOUD    4
#define MP_TRACE   3
#define MP_INFO    2
#define MP_WARNING 1
#define MP_ERROR   0

extern INT MPDebugLevel;*/

#ifndef min
#define    min(_a, _b)      (((_a) < (_b)) ? (_a) : (_b))
#endif

#ifndef max
#define    max(_a, _b)      (((_a) > (_b)) ? (_a) : (_b))
#endif

//--------------------------------------
// Utility macros
//--------------------------------------

#define MP_SET_FLAG(_M, _F)      ((_M)->Flags |= (_F))
#define MP_CLEAR_FLAG(_M, _F)    ((_M)->Flags &= ~(_F))
#define MP_TEST_FLAG(_M, _F)     ((((_M)->Flags & (_F)) != 0)?TRUE:FALSE)
#define MP_TEST_FLAGS(_M, _F)    ((((_M)->Flags & (_F)) == (_F))?TRUE:FALSE)

#define MP_IS_READY(_M)  (((_M)->Flags & \
         (fMP_DISCONNECTED | fMP_RESET_IN_PROGRESS | fMP_HALT_IN_PROGRESS | fMP_INIT_IN_PROGRESS | fMP_SURPRISE_REMOVED)) == 0)

#define MP_INC_REF(_A)  NdisInterlockedIncrement(&(_A)->RefCount)

#define MP_DEC_REF(_A) {\
                            NdisInterlockedDecrement(&(_A)->RefCount);\
                            ASSERT(_A->RefCount >= 0);\
                            if((_A)->RefCount == 0){\
                                NdisSetEvent(&(_A)->HaltEvent);\
                            }\
                        }

#define MP_GET_REF(_A)    ((_A)->RefCount)

typedef struct _MP_GLOBAL_DATA
{
    LIST_ENTRY      AdapterList;
    NDIS_SPIN_LOCK  Lock;
} MP_GLOBAL_DATA, *PMP_GLOBAL_DATA;

extern MP_GLOBAL_DATA GlobalData;

#include <pshpack1.h>

typedef struct _ETH_HEADER
{
    UCHAR       DstAddr[ETH_LENGTH_OF_ADDRESS];
    UCHAR       SrcAddr[ETH_LENGTH_OF_ADDRESS];
    USHORT      EthType;
} ETH_HEADER, *PETH_HEADER;

#include <poppack.h>

// TCB (Transmit Control Block)
typedef struct _TCB
{
    LIST_ENTRY              List; // This must be the first entry
    LONG                    Ref;
    PVOID                   Adapter;
    WDFREQUEST              Request;
    PMDL                    Mdl;
    PNDIS_BUFFER            Buffer;
    PNDIS_PACKET            OrgSendPacket;
    PUCHAR                  pData;
    ULONG                   ulSize;
    UCHAR                   Data[NIC_BUFFER_SIZE];
} TCB, *PTCB;

// RCB (Receive Control Block)
typedef struct _RCB
{
    LIST_ENTRY              List; // This must be the first entry
    LONG                    Ref;
    PVOID                   Adapter;
    WDFREQUEST              Request;
    PNDIS_BUFFER            Buffer;
    PNDIS_PACKET            Packet;
    PUCHAR                  pData;
    ULONG                   ulSize;
    UCHAR                   Data[NIC_BUFFER_SIZE];
} RCB, *PRCB;

typedef struct _MP_ADAPTER
{
    LIST_ENTRY              List;
    LONG                    RefCount;
    NDIS_SPIN_LOCK          Lock;
    NDIS_EVENT              InitEvent;
    NDIS_EVENT              HaltEvent;

 
    //
    // Keep track of various device objects.
    //
    PDEVICE_OBJECT          Pdo;
    PDEVICE_OBJECT          Fdo;
    PDEVICE_OBJECT          NextDeviceObject;
    WDFDEVICE               WdfDevice;
    NDIS_HANDLE             AdapterHandle;
    WCHAR                   AdapterName[NIC_ADAPTER_NAME_SIZE];
    WCHAR                   AdapterDesc[NIC_ADAPTER_NAME_SIZE];
    ULONG                   Flags;
    UCHAR                   PermanentAddress[ETH_LENGTH_OF_ADDRESS];
    UCHAR                   CurrentAddress[ETH_LENGTH_OF_ADDRESS];
	UCHAR                   OtherAddress[ETH_LENGTH_OF_ADDRESS];
    WDFWORKITEM             WorkItemForNdisRequest;
    BOOLEAN                 Promiscuous;
    BOOLEAN                 RequestPending;
    BOOLEAN                 ResetPending;
    BOOLEAN                 IsHardwareDevice;
    BOOLEAN                 IsTargetSupportChainedMdls;
    //
    // Variables to track resources for the send operation
    //
    NDIS_HANDLE             SendBufferPoolHandle;
    LIST_ENTRY              SendFreeList;
    LIST_ENTRY              SendWaitList;
    LIST_ENTRY              SendBusyList;
    PUCHAR                  TCBMem;
    LONG                    nBusySend;
    NDIS_SPIN_LOCK          SendLock;

    //
    // Variables to track resources for the Receive operation
    //
    LIST_ENTRY              RecvFreeList;
    LIST_ENTRY              RecvBusyList;
    NDIS_SPIN_LOCK          RecvLock;
    LONG                    nBusyRecv;
    NDIS_HANDLE             RecvPacketPoolHandle;
    NDIS_HANDLE             RecvBufferPoolHandle;
    PUCHAR                  RCBMem;
    WDFWORKITEM             ReadWorkItem;
    WDFWORKITEM             ExecutiveCallbackWorkItem;
    LONG                    IsReadWorkItemQueued;
    //
    // Packet Filter and look ahead size.
    //
    ULONG                   PacketFilter;
    ULONG                   ulLookAhead;
    ULONG                   ulLinkSpeed;

    // multicast list
    ULONG                   ulMCListSize;
    UCHAR                   MCList[NIC_MAX_MCAST_LIST][ETH_LENGTH_OF_ADDRESS];

    // Packet counts
    ULONG64                 GoodTransmits;
	ULONG64                 GoodTransmitsBytes;
    ULONG64                 GoodReceives;
	ULONG64                 GoodReceivesBytes;
    ULONG                   NumTxSinceLastAdjust;

    // Count of transmit errors
    ULONG                   TxAbortExcessCollisions;
    ULONG                   TxLateCollisions;
    ULONG                   TxDmaUnderrun;
    ULONG                   TxLostCRS;
    ULONG                   TxOKButDeferred;
    ULONG                   OneRetry;
    ULONG                   MoreThanOneRetry;
    ULONG                   TotalRetries;
    ULONG                   TransmitFailuresOther;

    // Count of receive errors
    ULONG                   RcvCrcErrors;
    ULONG                   RcvAlignmentErrors;
    ULONG                   RcvResourceErrors;
    ULONG                   RcvDmaOverrunErrors;
    ULONG                   RcvCdtFrames;
    ULONG                   RcvRuntErrors;

    //
    // Talking to NDISPROT protocol
    //
    HANDLE                  FileHandle;
    PFILE_OBJECT            FileObject;
    WDFIOTARGET             IoTarget;
    UCHAR                   PhyNicMacAddress[ETH_LENGTH_OF_ADDRESS];
    PCALLBACK_OBJECT        CallbackObject;
    PVOID                   CallbackRegisterationHandle;
    WDFREQUEST              StatusIndicationRequest;
    NDISPROT_INDICATE_STATUS NdisProtIndicateStatus;
    WDFMEMORY               WdfStatusIndicationBuffer;
    //
    // Statistic for debugging & validation purposes
    //
    ULONG                   nReadsPosted;
    ULONG                   nReadsCompletedWithError;
    ULONG                   nPacketsIndicated;
    ULONG                   nPacketsReturned;
    ULONG                   nBytesRead;
    ULONG                   nPacketsArrived;
    ULONG                   nWritesPosted;
    ULONG                   nWritesCompletedWithError;
    ULONG                   nBytesArrived;
    ULONG                   nBytesWritten;
    ULONG                   nReadWorkItemScheduled;
} MP_ADAPTER, *PMP_ADAPTER;


typedef struct _WDF_DEVICE_INFO{

    PMP_ADAPTER Adapter;

} WDF_DEVICE_INFO, *PWDF_DEVICE_INFO;

WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(WDF_DEVICE_INFO, GetWdfDeviceInfo)

//
// This structure is used to send requests to the WDM
// driver at PASSIVE_LEVEL.
//
typedef struct _WDM_NDIS_REQUEST
{
    PMP_ADAPTER         Adapter;
    NDIS_OID            Oid;
    NDIS_REQUEST_TYPE   RequestType;
    PVOID               InformationBuffer;
    ULONG               InformationBufferLength;
    PULONG              BytesReadOrWritten;
    PULONG              BytesNeeded;

} WDM_NDIS_REQUEST, *PWDM_NDIS_REQUEST;

WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(WDM_NDIS_REQUEST,
                                    GetContexOfNdisRequestWorkItem)


#define NDIS_STATUS_TO_NT_STATUS(_NdisStatus, _pNtStatus)                           \
{                                                                                   \
    /*                                                                              \
     *  The following NDIS status codes map directly to NT status codes.            \
     */                                                                             \
    if (((NDIS_STATUS_SUCCESS == (_NdisStatus)) ||                                  \
        (NDIS_STATUS_PENDING == (_NdisStatus)) ||                                   \
        (NDIS_STATUS_BUFFER_OVERFLOW == (_NdisStatus)) ||                           \
        (NDIS_STATUS_FAILURE == (_NdisStatus)) ||                                   \
        (NDIS_STATUS_RESOURCES == (_NdisStatus)) ||                                 \
        (NDIS_STATUS_NOT_SUPPORTED == (_NdisStatus))))                              \
    {                                                                               \
        *(_pNtStatus) = (NTSTATUS)(_NdisStatus);                                    \
    }                                                                               \
    else if (NDIS_STATUS_BUFFER_TOO_SHORT == (_NdisStatus))                         \
    {                                                                               \
        /*                                                                          \
         *  The above NDIS status codes require a little special casing.            \
         */                                                                         \
        *(_pNtStatus) = STATUS_BUFFER_TOO_SMALL;                                    \
    }                                                                               \
    else if (NDIS_STATUS_INVALID_LENGTH == (_NdisStatus))                           \
    {                                                                               \
        *(_pNtStatus) = STATUS_INVALID_BUFFER_SIZE;                                 \
    }                                                                               \
    else if (NDIS_STATUS_INVALID_DATA == (_NdisStatus))                             \
    {                                                                               \
        *(_pNtStatus) = STATUS_INVALID_PARAMETER;                                   \
    }                                                                               \
    else if (NDIS_STATUS_ADAPTER_NOT_FOUND == (_NdisStatus))                        \
    {                                                                               \
        *(_pNtStatus) = STATUS_NO_MORE_ENTRIES;                                     \
    }                                                                               \
    else if (NDIS_STATUS_ADAPTER_NOT_READY == (_NdisStatus))                        \
    {                                                                               \
        *(_pNtStatus) = STATUS_DEVICE_NOT_READY;                                    \
    }                                                                               \
    else                                                                            \
    {                                                                               \
        *(_pNtStatus) = STATUS_UNSUCCESSFUL;                                        \
    }                                                                               \
}

#define NT_STATUS_TO_NDIS_STATUS(_NtStatus, _pNdisStatus)                           \
{                                                                                   \
    /*                                                                              \
     *  The following NDIS status codes map directly to NT status codes.            \
     */                                                                             \
    if (((STATUS_SUCCESS == (_NtStatus)) ||                                  \
        (STATUS_PENDING == (_NtStatus)) ||                                   \
        (STATUS_BUFFER_OVERFLOW == (_NtStatus)) ||                           \
        (STATUS_UNSUCCESSFUL == (_NtStatus)) ||                                   \
        (STATUS_INSUFFICIENT_RESOURCES == (_NtStatus)) ||                                 \
        (STATUS_NOT_SUPPORTED == (_NtStatus))))                              \
    {                                                                               \
        *(_pNdisStatus) = (NDIS_STATUS)(_NtStatus);                                    \
    }                                                                               \
    else if (STATUS_BUFFER_TOO_SMALL == (_NtStatus))                         \
    {                                                                               \
        /*                                                                          \
         *  The above NDIS status codes require a little special casing.            \
         */                                                                         \
        *(_pNdisStatus) = NDIS_STATUS_BUFFER_TOO_SHORT;                            \
    }                                                                               \
    else if (STATUS_INVALID_BUFFER_SIZE == (_NtStatus))                           \
    {                                                                               \
        *(_pNdisStatus) = NDIS_STATUS_INVALID_LENGTH;                                 \
    }                                                                               \
    else if (STATUS_INVALID_PARAMETER == (_NtStatus))                             \
    {                                                                               \
        *(_pNdisStatus) = NDIS_STATUS_INVALID_DATA;                                   \
    }                                                                               \
    else if (STATUS_NO_MORE_ENTRIES == (_NtStatus))                        \
    {                                                                               \
        *(_pNdisStatus) = NDIS_STATUS_ADAPTER_NOT_FOUND;                                     \
    }                                                                               \
    else if (STATUS_DEVICE_NOT_READY == (_NtStatus))                        \
    {                                                                               \
        *(_pNdisStatus) = NDIS_STATUS_ADAPTER_NOT_READY;                                    \
    }                                                                               \
    else                                                                            \
    {                                                                               \
        *(_pNdisStatus) = NDIS_STATUS_FAILURE;                                        \
    }                                                                               \
}




//--------------------------------------
// Miniport routines
//--------------------------------------

DRIVER_INITIALIZE DriverEntry;

NDIS_STATUS
MPInitialize(
    OUT PNDIS_STATUS OpenErrorStatus,
    OUT PUINT SelectedMediumIndex,
    IN PNDIS_MEDIUM MediumArray,
    IN UINT MediumArraySize,
    IN NDIS_HANDLE MiniportAdapterHandle,
    IN NDIS_HANDLE WrapperConfigurationContext);


BOOLEAN
MPCheckForHang(
    IN NDIS_HANDLE MiniportAdapterContext);

VOID
MPHalt(
    IN  NDIS_HANDLE MiniportAdapterContext);

NDIS_STATUS
MPReset(
    OUT PBOOLEAN AddressingReset,
    IN  NDIS_HANDLE MiniportAdapterContext
    );

VOID
MPUnload(
    IN  PDRIVER_OBJECT  DriverObject
    );

NDIS_STATUS
MPQueryInformation(
    IN NDIS_HANDLE MiniportAdapterContext,
    IN NDIS_OID Oid,
    IN PVOID InformationBuffer,
    IN ULONG InformationBufferLength,
    OUT PULONG BytesWritten,
    OUT PULONG BytesNeeded);


VOID
MPReturnPacket(
    IN NDIS_HANDLE  MiniportAdapterContext,
    IN PNDIS_PACKET Packet);

VOID
MPSendPackets(
    IN  NDIS_HANDLE             MiniportAdapterContext,
    IN  PPNDIS_PACKET           PacketArray,
    IN  UINT                    NumberOfPackets);

NDIS_STATUS
MPSetInformation(
    IN NDIS_HANDLE MiniportAdapterContext,
    IN NDIS_OID Oid,
    IN PVOID InformationBuffer,
    IN ULONG InformationBufferLength,
    OUT PULONG BytesRead,
    OUT PULONG BytesNeeded);

VOID
MPShutdown(
    IN  NDIS_HANDLE MiniportAdapterContext);

DRIVER_DISPATCH PtDispatch;

NTSTATUS
PtDispatch(
    IN PDEVICE_OBJECT            DeviceObject,
    IN PIRP                      Irp
    );

NDIS_STATUS
PtRegisterDevice(
    VOID
    );

NDIS_STATUS
PtDeregisterDevice(
    VOID
   );


BOOLEAN
NICCopyPacket(
    PMP_ADAPTER Adapter,
    PTCB pTCB,
    PNDIS_PACKET Packet);


VOID
NICFreeRecvPacket(
    PMP_ADAPTER Adapter,
    PNDIS_PACKET Packet);

VOID
NICFreeSendTCB(
    IN PMP_ADAPTER Adapter,
    IN PTCB pTCB);


VOID
NICFreeQueuedSendPackets(
    PMP_ADAPTER Adapter
    );


NDIS_STATUS
NICInitializeAdapter(
    IN PMP_ADAPTER Adapter,
    IN  NDIS_HANDLE  WrapperConfigurationContext
);

NDIS_STATUS
NICAllocAdapter(
    PMP_ADAPTER *Adapter
    );

VOID
NICFreeAdapter(
    PMP_ADAPTER Adapter
    );

NDIS_STATUS
NICAllocSendResources(
    PMP_ADAPTER Adapter);

NDIS_STATUS
NICAllocRecvResources(
    PMP_ADAPTER Adapter);

VOID
NICFreeSendResources(
    PMP_ADAPTER Adapter);

VOID
NICFreeRecvResources(
    PMP_ADAPTER Adapter);

VOID
NICAttachAdapter(
    PMP_ADAPTER Adapter
    );

VOID
NICDetachAdapter(
    PMP_ADAPTER Adapter
    );

NDIS_STATUS
NICReadRegParameters(
    PMP_ADAPTER Adapter,
    NDIS_HANDLE ConfigurationHandle);

NDIS_STATUS
NICGetStatsCounters(
    PMP_ADAPTER Adapter,
    NDIS_OID Oid,
    ULONG_PTR pCounter);

NDIS_STATUS
NICSetPacketFilter(
    IN PMP_ADAPTER Adapter,
    IN ULONG PacketFilter);

NDIS_STATUS
NICSetMulticastList(
    IN PMP_ADAPTER              Adapter,
    IN PVOID                    InformationBuffer,
    IN ULONG                    InformationBufferLength,
    OUT PULONG                  pBytesRead,
    OUT PULONG                  pBytesNeeded
    );

ULONG
NICGetMediaConnectStatus(
    PMP_ADAPTER Adapter);

#ifdef NDIS51_MINIPORT

VOID
MPCancelSendPackets(
    IN  NDIS_HANDLE     MiniportAdapterContext,
    IN  PVOID           CancelId
    );

VOID MPPnPEventNotify(
    IN  NDIS_HANDLE             MiniportAdapterContext,
    IN  NDIS_DEVICE_PNP_EVENT   PnPEvent,
    IN  PVOID                   InformationBuffer,
    IN  ULONG                   InformationBufferLength
    );

#endif

PCHAR
DbgGetOidName (
    ULONG oid
    );

VOID
NICInitWorkItemCallback(
    WDFWORKITEM WorkItem
    );

NTSTATUS
NICInitAdapterWorker(
    PMP_ADAPTER Adapter
    ) ;

VOID
NICPostReadsWorkItemCallBack(
    WDFWORKITEM WorkItem
    );

NTSTATUS
NICMakeSynchronousIoctl(
    IN WDFIOTARGET          IoTarget,
    IN PFILE_OBJECT         FileObject,
    IN ULONG                IoctlControlCode,
    IN OUT PVOID            InputBuffer,
    IN ULONG                InputBufferLength,
    IN OUT PVOID            OutputBuffer,
    IN ULONG                OutputBufferLength,
    OUT PULONG              BytesReadOrWritten
    );

VOID
NICMakeSynchronousIoCtlCompletion(
    IN WDFREQUEST                  Request,
    IN WDFIOTARGET                 Target,
    PWDF_REQUEST_COMPLETION_PARAMS CompletionParams,
    IN WDFCONTEXT                  Context
    );

NTSTATUS
NICPostWriteRequest(
    PMP_ADAPTER Adapter,
    PTCB    pTCB
    );

VOID
NICWriteRequestCompletion(
    IN WDFREQUEST                  Request,
    IN WDFIOTARGET                 Target,
    PWDF_REQUEST_COMPLETION_PARAMS CompletionParams,
    IN WDFCONTEXT                  Context
    );

NTSTATUS
NICPostReadRequest(
    PMP_ADAPTER Adapter,
    PRCB    pRCB
    );

/*
VOID
NICReadRequestCompletion(
    IN WDFREQUEST                  Request,
    IN WDFIOTARGET                 Target,
    PWDF_REQUEST_COMPLETION_PARAMS CompletionParams,
    IN WDFCONTEXT                  Context
    );*/
EVT_WDF_REQUEST_COMPLETION_ROUTINE  NICReadRequestCompletion;

VOID
NICFreeRCB(
    IN PRCB pRCB
    );

NDIS_STATUS
NICSendRequest(
    IN PMP_ADAPTER                  Adapter,
    IN NDIS_REQUEST_TYPE            RequestType,
    IN NDIS_OID                     Oid,
    IN PVOID                        InformationBuffer,
    IN ULONG                        InformationBufferLength,
    OUT PULONG                      BytesReadOrWritten,
    OUT PULONG                      BytesNeeded
    );

NDIS_STATUS
NICForwardRequest(
    IN PMP_ADAPTER                  Adapter,
    IN NDIS_REQUEST_TYPE            RequestType,
    IN NDIS_OID                     Oid,
    IN PVOID                        InformationBuffer,
    IN ULONG                        InformationBufferLength,
    OUT PULONG                      BytesReadOrWritten,
    OUT PULONG                      BytesNeeded
    );

VOID
NICRequestWorkItemCallback(
    WDFWORKITEM WorkItem
    );

VOID
NICIndicateReceivedPacket(
    IN PRCB             pRCB,
    IN ULONG            BytesRead
    );

BOOLEAN
NICIsPacketAcceptable(
    IN PMP_ADAPTER Adapter,
    IN PUCHAR   pDstMac
    );

BOOLEAN
NICIsMulticastMatch(
    IN PMP_ADAPTER                  Adapter,
    IN PUCHAR                       pDstMac
    );


#ifdef INTERFACE_WITH_NDISPROT

VOID
NICUnregisterExCallback(
    IN PMP_ADAPTER Adapter
    );

BOOLEAN
NICRegisterExCallback(
    PMP_ADAPTER Adapter
    );

VOID
NICExCallback(
    PVOID   CallBackContext,
    PVOID   Source,
    PVOID   AreYouThere
    );

NTSTATUS
NICOpenNdisProtocolInterface(
    PMP_ADAPTER Adapter
    );

#else

#define NICUnregisterExCallback(Adapter)
#define NICRegisterExCallback(Adapter) TRUE
#define NICExCallback()

#endif


NTSTATUS
NICPostAsynchronousStatusIndicationRequest(
    IN PMP_ADAPTER Adapter
    );

VOID
NICStatusIndicationRequestCompletionRoutine(
    IN WDFREQUEST                  Request,
    IN WDFIOTARGET                 Target,
    PWDF_REQUEST_COMPLETION_PARAMS CompletionParams,
    IN WDFCONTEXT                  Context
    );

#endif    // NDISWDM_H



