/*++

Copyright (c) 2009	Microsoft Corporation

Module Name:
	DeviceIo.c

Abstract:
	Handle Device IO Control from User Application or Upper Layers

Revision History:
	Who 		When		What						
	--------	--------	-----------------------------------------------
	Chen Tian	16-03-09	Wrtitten For NetFPGA2 Card in Microsoft Research Asia
Notes:

--*/

#include "..\inc\precomp.h"
#include "..\inc\Device.h"
#include "..\inc\DeviceIo.h"
#include "..\inc\PCIRequest.h"
#include "..\inc\dbg.h"
#include "..\inc\nic_req.h"


#ifdef ALLOC_PRAGMA
#endif


NTSTATUS
StartMac(IN PDEVICE_EXTENSION pDevExt)
{
	NTSTATUS Status = STATUS_SUCCESS;

	do 
	{
	} while (FALSE);

	return Status;
}

NTSTATUS
StopMac(IN PDEVICE_EXTENSION pDevExt)
{
	NTSTATUS Status = STATUS_SUCCESS;

	do 
	{
	} while (FALSE);

	return Status;
}


NTSTATUS
GetMacMmManager(IN PDEVICE_EXTENSION pDevExt, OUT PULONG pMacMmManager)
{
	NTSTATUS Status = STATUS_SUCCESS;

	do 
	{
		
	} while (FALSE);

	return Status;
}


// IoDeviceControl is called for IRP_MJ_DEVICE_CONTROL
VOID PCIEDeviceIoCtrl(
	IN WDFQUEUE		Queue,
	IN WDFREQUEST	Request,
	IN size_t			OutputBufferLength,
	IN size_t			InputBufferLength,
	IN ULONG		IoControlCode
	)
/*++
Routine Description:

This event is called when the framework receives IRP_MJ_DEVICE_CONTROL
requests from the system.

Arguments:

Queue - Handle to the framework queue object that is associated
with the I/O request.
Request - Handle to a framework request object.

OutputBufferLength - ReturnLength of the request's output buffer,
if an output buffer is available.
InputBufferLength - ReturnLength of the request's input buffer,
if an input buffer is available.

IoControlCode - the driver-defined or system-defined I/O control code
(IOCTL) that is associated with the request.
Return Value:

VOID

--*/
{
	NTSTATUS					Status = STATUS_SUCCESS;
	PDEVICE_EXTENSION			pDevExt;
	
	PVOID						pInputBuffer;
	PVOID						pOutputBuffer;
	SIZE_T						InputBufferSize;
	SIZE_T						OutputBufferSize;
	WDF_REQUEST_PARAMETERS	params;
	
	ULONG						ReturnLength;

	DBGPRINT(MP_LOUD, ("---> PCIEDeviceIoCtrl"));

	//
	// Get the device extension.
	//
	pDevExt = PCIEGetDeviceContext(WdfIoQueueGetDevice( Queue ));


	if (InputBufferLength > 0)
	{
		Status = WdfRequestRetrieveInputBuffer(
					Request, 
					InputBufferLength,
					&pInputBuffer, 
					&InputBufferSize
					);
		
		if( !NT_SUCCESS(Status) )
		{
			WdfRequestComplete(Request, Status);
			return;
		}
	}
	else
	{
		pInputBuffer = NULL;
		InputBufferSize = 0;
	}

	if (OutputBufferLength > 0)
	{
		Status = WdfRequestRetrieveOutputBuffer(
					Request, 
					OutputBufferLength, 
					&pOutputBuffer, 
					&OutputBufferSize
					);

		if( !NT_SUCCESS(Status) )
		{
			WdfRequestComplete(Request, Status);
			return;
		}
	}
	else
	{
		pOutputBuffer = NULL;
		OutputBufferSize = 0;
	}

	WDF_REQUEST_PARAMETERS_INIT(&params);

	WdfRequestGetParameters(
		Request,
		&params
		);

	//
	// Handle this request's specific code.
	//
	switch ( IoControlCode )
	{
	case IOCTL_BUDDY_CONNECT:
		if ( OutputBufferLength < sizeof(ULONG))
		{
			Status = STATUS_INVALID_DEVICE_REQUEST;
			ReturnLength = 0;
			break;
		}

		Status = GetMacMmManager(pDevExt, pOutputBuffer);
		ReturnLength = sizeof(ULONG);

		break;

	case IOCTL_START_MAC:
		Status = StartMac(pDevExt);
		ReturnLength = 0;

		break;

	case IOCTL_STOP_MAC:
		Status = StopMac(pDevExt);
		ReturnLength = 0;

		break;

	case IOCTL_READ_REG:
		// make sure the size is larger than that of USHORT
		if ( OutputBufferLength < sizeof(ULONG)) 
		{
			Status = STATUS_INVALID_DEVICE_REQUEST;
			ReturnLength = 0;
			break;
		}
		FPGAReadRegister( pDevExt, pInputBuffer, pOutputBuffer);
		ReturnLength = OutputBufferLength;
		
		break;

	case IOCTL_WRITE_REG:
		Status = FPGAWriteRegister( pDevExt, pInputBuffer, InputBufferLength);
		if ( Status == STATUS_INVALID_DEVICE_REQUEST) 
		{
			ReturnLength = 0;
			break;
		}
		ReturnLength = 0;
		break;

	case IOCTL_GET_PCI_INFO:
		// help function, get all the pci slot information to the user space
		if ( OutputBufferLength < sizeof(PCI_COMMON_CONFIG) )
		{
			Status = STATUS_INVALID_DEVICE_REQUEST;
			ReturnLength = 0;
			break;
		}
		PCIEEvtGetPCIInfo ( pDevExt, (PPCI_COMMON_CONFIG)pOutputBuffer);
		ReturnLength = sizeof (PCI_COMMON_CONFIG);
		break;

	case IOCTL_SET_PCI_INFO:
		// help function, get all the pci slot information to the user space
		if ( InputBufferLength < sizeof(PCI_COMMON_CONFIG) ) 
		{
			Status = STATUS_INVALID_DEVICE_REQUEST;
			ReturnLength = 0;
			break;
		}
		PCIEEvtSetPCIInfo ( pDevExt, (PPCI_COMMON_CONFIG)pInputBuffer);
		ReturnLength = 0;
		break;

	// These IOCTL code are used by the NDIS miniport driver above
	case IOCTL_NDISPROT_QUERY_OID_VALUE:

		ASSERT((IoControlCode & 0x3) == METHOD_BUFFERED);
		ReturnLength = NICHandleQueryOidRequest(
			Queue,
			Request,
			pDevExt,
			pOutputBuffer,
			OutputBufferLength,
			&params
			);
		break;

	case IOCTL_NDISPROT_SET_OID_VALUE:

		ASSERT((IoControlCode & 0x3) == METHOD_BUFFERED);

		DBGPRINT(MP_WARN,( "SET_OID"));
		NICHandleSetOidRequest(
			Queue,
			Request,
			pDevExt,
			pInputBuffer,
			InputBufferLength,
			&params
			);

		break;

	case IOCTL_NDISPROT_INDICATE_STATUS:

		Status = WdfRequestForwardToIoQueue(Request, pDevExt->PendingIoctlQueue);
		
		if(!NT_SUCCESS(Status))
		{
			DBGPRINT(MP_INFO,( "WdfRequestForwardToIoQueue failed 0x%x\n", Status));
			ReturnLength=0;
			break;
		}
		ReturnLength=0;
		// this request will be pending until the media status changes
		return;

	default:
		Status = STATUS_INVALID_DEVICE_REQUEST;
		ReturnLength = 0;

		break;
	}

	if (Status != STATUS_PENDING)
	{
		WdfRequestCompleteWithInformation(Request, Status, ReturnLength);
	}

	DBGPRINT(MP_LOUD, ("<--- PCIEDeviceIoCtrl"));

}


VOID PCIEInternalDeviceControl(
	IN WDFQUEUE		Queue,
	IN WDFREQUEST	Request,
	IN size_t			OutputBufferLength,
	IN size_t			InputBufferLength,
	IN ULONG		IoControlCode
	)
{
	NTSTATUS			Status = STATUS_SUCCESS;
	PDEVICE_EXTENSION	pDevExt;

	PVOID				pInputBuffer;
	PVOID				pOutputBuffer;
	SIZE_T				InputBufferSize;
	SIZE_T				OutputBufferSize;

	ULONG				ReturnLength;
	PIRP					Irp = NULL;

	DBGPRINT(MP_INFO, ("---> Internal Device Control"));

	//
	// Get the device extension.
	//
	pDevExt = PCIEGetDeviceContext(WdfIoQueueGetDevice( Queue ));

	if ( InputBufferLength > 0)
	{
		Status = WdfRequestRetrieveInputBuffer(
					Request, 
					InputBufferLength, 
					&pInputBuffer, 
					&InputBufferSize
					);
		
		if( !NT_SUCCESS(Status) )
		{
			WdfRequestComplete(Request, Status);
			return;
		}
	}
	else
	{
		pInputBuffer = NULL;
		InputBufferSize = 0;
	}

	if (OutputBufferLength > 0)
	{
		Status = WdfRequestRetrieveOutputBuffer(
					Request, 
					OutputBufferLength, 
					&pOutputBuffer, 
					&OutputBufferSize
					);
		
		if( !NT_SUCCESS(Status) )
		{
			WdfRequestComplete(Request, Status);
			return;
		}
	}
	else
	{
		pOutputBuffer = NULL;
		OutputBufferSize = 0;
	}

	//
	// Handle this request's specific code.
	//
	switch (IoControlCode)
	{
	default:
		Status = STATUS_INVALID_DEVICE_REQUEST;
		ReturnLength = 0;
		break;
	}

	if (Status != STATUS_PENDING)
	{
		WdfRequestCompleteWithInformation(Request, Status, ReturnLength);
	}

	DBGPRINT(MP_INFO, ("<--- Internal Device Control"));

}