/*++

Copyright (c) 2009	Microsoft Corporation

Module Name:
	Driver.c

Abstract:
	Entry of the Driver

Revision History:
	Who 		When		What						
	--------	--------	-----------------------------------------------
	Chen Tian	16-03-09	Wrtitten For NetFPGA2 Card in Microsoft Research Asia
Notes:

--*/

#include "..\inc\precomp.h"
#include "..\inc\Driver.h"
#include "..\inc\Device.h"
#include "..\inc\dbg.h"

#ifdef ALLOC_PRAGMA
#pragma alloc_text (INIT, DriverEntry)
#pragma alloc_text (PAGE, PCIEEvtDriverContextCleanup)
#endif

/*++

Routine Description:

Driver initialization entry point.
This entry point is called directly by the I/O system.

Arguments:

DriverObject - pointer to the driver object

RegistryPath - pointer to a unicode string representing the path,
to driver-specific key in the registry.

Return Value:

NTSTATUS	- if the status value is not STATUS_SUCCESS,
the driver will get unloaded immediately.

--*/
NTSTATUS
DriverEntry(
	IN PDRIVER_OBJECT		DriverObject,
	IN PUNICODE_STRING		RegistryPath
	)
{
	NTSTATUS				Status = STATUS_SUCCESS;
	WDF_DRIVER_CONFIG		Config;
	WDF_OBJECT_ATTRIBUTES	Attributes;

	DBGPRINT(MP_TRACE, ("---> DriverEntry"));

	//
	// Initialize the Driver Config structure.
	//
	WDF_DRIVER_CONFIG_INIT( &Config, PCIEEvtDeviceAdd );

	//
	// Register a cleanup callback so that we can call WPP_CLEANUP when
	// the framework driver object is deleted during driver unload.
	//
	WDF_OBJECT_ATTRIBUTES_INIT(&Attributes);
	Attributes.EvtCleanupCallback = PCIEEvtDriverContextCleanup;

	Status = WdfDriverCreate( 
		DriverObject,
		RegistryPath,
		&Attributes,
		&Config,
		WDF_NO_HANDLE
		);

	DBGPRINT(MP_INFO, ("<--- DriverEntry"));

	return Status;
}

VOID
PCIEEvtDriverContextCleanup(
	IN WDFDRIVER Driver
	)
/*++
Routine Description:

	Free all the resources allocated in DriverEntry.

Arguments:

	Driver - handle to a WDF Driver object.

Return Value:

	VOID.

--*/
{
	DBGPRINT(MP_TRACE, ("---> PCIEEvtDriverContextCleanup"));
	UNREFERENCED_PARAMETER(Driver);

	PAGED_CODE ();

	DBGPRINT(MP_TRACE, ("<--- PCIEEvtDriverContextCleanup"));
}