/*++

Copyright (c) 2009	Microsoft Corporation

Module Name:
	PCIRequest.c

Abstract:
	Interface with PCI Bus

Revision History:
	Who 		When		What						
	--------	--------	-----------------------------------------------
	Chen Tian	16-03-09	Wrtitten For NetFPGA2 Card in Microsoft Research Asia
Notes:

--*/

#include "..\inc\precomp.h"
#include "ntddk.h"
#include "..\inc\dbg.h"
#include "..\inc\Device.h"
#include "..\inc\nf2.h"

#define PCI_MAX_BUSES 3

// globale variables to maintain the bus number and slot number of NetFPGA
static int gBusNumer=0;
static PCI_SLOT_NUMBER gSlotNumber;

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
NTSTATUS
FPGAWriteRegister (
	 IN	PDEVICE_EXTENSION	pDevExt,
	 IN	PVOID				pRegister,
	 IN size_t					length
	)

{
	NTSTATUS		  status = STATUS_SUCCESS;
	PRegDes			  pReg;

	DBGPRINT(MP_INFO, ("---> FPGAWriteRegister"));

	// make sure the parameter is the size of struct RegDes
	if ( length != sizeof(RegDes)) 
	{
		status = STATUS_INVALID_DEVICE_REQUEST;
		return status;
	}

	// cast the input data to RegDes
	pReg = (PRegDes) pRegister;

	DBGPRINT(MP_LOUD, ("Address %x, Value %x ", pReg->Address, pReg->Value));

	WriteRegisterUlong (pDevExt, pReg->Address, pReg->Value);

	DBGPRINT(MP_INFO, ("<--- FPGAWriteRegister"));

	return status;

}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
VOID
FPGAReadRegister(
	IN	PDEVICE_EXTENSION	pDevExt,
	IN	PVOID				pInputBuffer,
	OUT PVOID				pOutBuffer
	)
{
	PRegDes			  pRegIn;
	PRegDes			  pRegOut;

	DBGPRINT(MP_INFO, ("---> FPGAReadRegister"));

	// cast the input data to RegDes, Normally they are the same
	pRegIn = (PRegDes) pInputBuffer;
	pRegOut = (PRegDes) pOutBuffer;
	pRegOut->Value = ReadRegisterUlong (pDevExt, pRegIn->Address);

	DBGPRINT(MP_INFO, ("<--- FPGAReadRegister"));

	return;
}

NTSTATUS
HtsGenericCompletion (
	IN PDEVICE_OBJECT	DeviceObject,
	IN PIRP 				Irp,
	IN PVOID				Context
	)

{
	UNREFERENCED_PARAMETER (DeviceObject);	  
	KeSetEvent ((PKEVENT) Context, IO_NO_INCREMENT, FALSE);
	return STATUS_MORE_PROCESSING_REQUIRED; // Keep this IRP
}


NTSTATUS
HtsReadWriteConfig(
	PDEVICE_OBJECT 	DeviceObject,
	ULONG 			WhichSpace,
	PVOID 			Buffer,
	ULONG 			Offset,
	ULONG 			Length,
	BOOLEAN 		ReadConfig)
{

	NTSTATUS Status;

	KEVENT Event;

	PIO_STACK_LOCATION IoStack;

	PIRP Irp = IoAllocateIrp(DeviceObject->StackSize, FALSE);

	if (!Irp) 
	{
		return STATUS_INSUFFICIENT_RESOURCES;
	}
	
	Irp->IoStatus.Status = STATUS_NOT_SUPPORTED;

	KeInitializeEvent(&Event, NotificationEvent, FALSE);

	IoSetCompletionRoutine(
		Irp,
		HtsGenericCompletion,
		&Event, 
		TRUE, 
		TRUE, 
		TRUE
		);

	IoStack = IoGetNextIrpStackLocation(Irp);
	IoStack->MajorFunction= IRP_MJ_PNP;
	IoStack->MinorFunction= ReadConfig ? IRP_MN_READ_CONFIG : IRP_MN_WRITE_CONFIG;
	IoStack->Parameters.ReadWriteConfig.WhichSpace = WhichSpace;
	IoStack->Parameters.ReadWriteConfig.Buffer = Buffer;
	IoStack->Parameters.ReadWriteConfig.Offset = Offset;
	IoStack->Parameters.ReadWriteConfig.Length = Length;

	if (ReadConfig) 
	{
		RtlZeroMemory(Buffer, Length);
	} 

	Status = IoCallDriver(DeviceObject, Irp); 

	if (Status == STATUS_PENDING) 
	{
		KeWaitForSingleObject(&Event, Executive, KernelMode, FALSE, NULL);
		Status = Irp->IoStatus.Status;
	}

	IoFreeIrp(Irp);

	return Status;
}


/**
	Read the PCI slot information
*/
void PCIEEvtGetPCIInfo ( 
 	IN	PDEVICE_EXTENSION		pDevExt,
 	OUT PPCI_COMMON_CONFIG 	pConfig) 
 {
	PCI_SLOT_NUMBER 		slotNumber;

	int busNumber, deviceNumber, functionNumber;

	int length=0;

	HtsReadWriteConfig(
		WdfDeviceWdmGetPhysicalDevice (pDevExt->Device), 
		PCI_WHICHSPACE_CONFIG, 
		pConfig,
		0, 
		sizeof(PCI_COMMON_CONFIG), 
		TRUE
		);
};

/**
	Write the PCI slot information
*/
void PCIEEvtSetPCIInfo (
	IN	PDEVICE_EXTENSION	pDevExt,
	IN PPCI_COMMON_CONFIG pConfig) 
{
	if (pConfig->VendorID != PCI_VENDOR_ID_STANFORD) 
	{
		DBGPRINT(MP_INFO, ("The configuration information is wrong..."));
		return;
	}

	HtsReadWriteConfig(
		WdfDeviceWdmGetPhysicalDevice (pDevExt->Device), 
		PCI_WHICHSPACE_CONFIG, 
		pConfig,
		0, 
		sizeof(PCI_COMMON_CONFIG), 
		FALSE);
};
