/*++

Copyright (c) 2009	Microsoft Corporation

Module Name:
	Read.c

Abstract:
	PCI read operations

Revision History:
	Who 		When		What						
	--------	--------	-----------------------------------------------
	Chen Tian	16-03-09	Wrtitten For NetFPGA2 Card in Microsoft Research Asia
Notes:

--*/

#include "..\inc\precomp.h"
#include "..\inc\dbg.h"
#include "..\inc\device.h"
#include "..\inc\nf2.h"
#include "..\inc\reg_defines.h"

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
VOID
PciDrvEvtIoRead(
	IN WDFQUEUE 	Queue,
	IN WDFREQUEST	Request,
	IN size_t			Length
	)
/*++

Routine Description:

	Called by the framework as soon as it receives a read request.
	If the device is not ready, fail the request.
	Otherwise get scatter-gather list for this request and send the
	packet to the hardware for DMA.

Arguments:

	Queue	   - Default queue handle
	Request    - Handle to the write request
	Lenght - Length of the data buffer associated with the request.
					 The default property of the queue is to not dispatch
				 zero lenght read & write requests to the driver and
				 complete is with status success. So we will never get
				 a zero length request.

Return Value:

--*/
{
	NTSTATUS				 status = STATUS_SUCCESS;
	PDEVICE_EXTENSION		pDevExt=NULL;
	PMDL					mdl;
	PVOID					virtualAddress;
	ULONG					length;

	DBGPRINT(MP_LOUD, ("---> PciDrvEvtIoRead"));

	DBGPRINT(MP_LOUD, ("<--- PciDrvEvtIoRead"));

	return;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
BOOLEAN
PCIEEvtProgramReadDma(
	IN	WDFDMATRANSACTION		Transaction,
	IN	WDFDEVICE				Device,
	IN	WDFCONTEXT				Context,
	IN	WDF_DMA_DIRECTION		Direction,
	IN	PSCATTER_GATHER_LIST	SgList
	)
/*++

Routine Description:

  The framework calls a driver's EvtProgramDma event callback function
  when the driver calls WdfDmaTransactionExecute and the system has
  enough map registers to do the transfer. The callback function must
  program the hardware to start the transfer. A single transaction
  initiated by calling WdfDmaTransactionExecute may result in multiple
  calls to this function if the buffer is too large and there aren't
  enough map registers to do the whole transfer.


Arguments:

Return Value:

--*/
{
	PDEVICE_EXTENSION	pDevExt;
	size_t				offset;
	ULONG				AddressLow;
	BOOLEAN				errors;
	ULONG				length;
	ULONG				i;

	UNREFERENCED_PARAMETER( Context );
	UNREFERENCED_PARAMETER( Direction );

	DBGPRINT(MP_INFO, ("---> PCIEEvtProgramReadDma"));

	//
	// Initialize locals
	//
	pDevExt = PCIEGetDeviceContext(Device);
	errors = FALSE;

	//
	// Get the number of bytes as the offset to the beginning of this
	// Dma operations transfer location in the buffer.
	//
	offset = WdfDmaTransactionGetBytesTransferred(Transaction);

	for (i=0; i < SgList->NumberOfElements; i++) 
	{
		//
		// Construct this DTE.
		//
		// NOTE: The LocalAddress is the offset into the SRAM from
		//		 where this Write will start.
		//
		AddressLow	= SgList->Elements[i].Address.LowPart;
		length		= SgList->Elements[i].Length;

		//
		// Increment the DmaTransaction length by this element length
		//
		offset += SgList->Elements[i].Length;

		/* Start the transfer */
		WriteRegisterUlong (pDevExt, CPCI_DMA_ADDR_I_REG, AddressLow);
		WriteRegisterUlong (pDevExt, CPCI_DMA_CTRL_I_REG, DMA_CTRL_OWNER);
		
	}

	DBGPRINT(MP_INFO, ("<--- PCIEEvtProgramReadDma"));

	return FALSE;
}

VOID
PCIEReadRequestComplete(
	IN WDFDMATRANSACTION	DmaTransaction,
	IN NTSTATUS 				Status
	)
/*++

Routine Description:

Arguments:

Return Value:

--*/
{
	UNREFERENCED_PARAMETER(DmaTransaction);
	UNREFERENCED_PARAMETER(Status);
}


