/*++

Copyright (c) 2009	Microsoft Corporation

Module Name:
	Nf2util.c

Abstract:
	Utility functions for nf2 driver, Revised From NetFPGA 2 Linux Kernel Driver

Revision History:
	Who 		When		What						
	--------	--------	-----------------------------------------------
	Chen Tian	16-03-09	Wrtitten For NetFPGA2 Card in Microsoft Research Asia
Notes:

--*/

#include "..\inc\Precomp.h"
#include "..\inc\Device.h"
#include "..\inc\nf2.h"
#include "..\inc\dbg.h"

/*
 * Reset the HW
 */
void NF2ResetAll ( PDEVICE_EXTENSION pDevExt)
{

	DBGPRINT(MP_WARN, ("---> NF2ResetAll <---"));
	/* Reset the CPCI */
	WriteRegisterUlong (pDevExt, CPCI_REG_RESET, RESET_CPCI);

	/* Reset the CNET */
	if (pDevExt->is_ctrl)
	{
		WriteRegisterUlong (pDevExt, CNET_REG_CTRL, CTRL_CNET_RESET);

		WriteRegisterUlong (pDevExt, CNET_REG_RESET,
			CNET_RESET_MAC_3 |
			CNET_RESET_MAC_2 |
			CNET_RESET_MAC_1 |
			CNET_RESET_MAC_0);

		/* Disable all MACs */
		WriteRegisterUlong (pDevExt, CNET_REG_ENABLE, 0);
	}

	/* Flush the writes */
	ReadRegisterUlong (pDevExt, 0);
}

/*
 * Reset the CPCI chip.
 *
 * Make sure to restore interrupts to their previous state
 */
void NF2ResetCPCI ( PDEVICE_EXTENSION pDevExt)
{
	ULONG intmask;

	DBGPRINT(MP_WARN, ("---> NF2ResetCPCI <---"));

	intmask = ReadRegisterUlong (pDevExt, CPCI_REG_INTERRUPT_MASK);
	WriteRegisterUlong (pDevExt, CPCI_REG_RESET, RESET_CPCI);
	WriteRegisterUlong (pDevExt, CPCI_REG_INTERRUPT_MASK, intmask);

	/* Flush the writes */
	ReadRegisterUlong (pDevExt, 0);
}

/*
 * Enable interrupts
 */
NTSTATUS
NF2InterruptEnable(
	IN WDFINTERRUPT  Interrupt,
	IN WDFDEVICE	 AssociatedDevice
	)

{
	PDEVICE_EXTENSION pDevExt;

	DBGPRINT(MP_TRACE, ("---> NF2InterruptEnable <---"));

	pDevExt = PCIEGetDeviceContext ( AssociatedDevice);
	WriteRegisterUlong (pDevExt, CPCI_REG_INTERRUPT_MASK, 0x00000000);
	
	/* Flush the writes */
	ReadRegisterUlong (pDevExt, 0);
	
	return STATUS_SUCCESS;
}

NTSTATUS
NF2InterruptDisable(
	IN WDFINTERRUPT  Interrupt,
	IN WDFDEVICE	 AssociatedDevice
	)
{

	PDEVICE_EXTENSION pDevExt;

	DBGPRINT(MP_INFO, ("---> NF2InterruptDisable <---"));

	pDevExt = PCIEGetDeviceContext ( AssociatedDevice);
	WriteRegisterUlong (pDevExt, CPCI_REG_INTERRUPT_MASK, 0xFFFFFFFF);
	
	/* Flush the writes */
	ReadRegisterUlong (pDevExt, 0);

	return STATUS_SUCCESS;
}
