/*++

Copyright (c) 2009	Microsoft Corporation

Module Name:
	Nic_defl.h

Abstract:
	Header file for kernel driver

Revision History:
	Who 		When		What						
	--------	--------	-----------------------------------------------
	Chen Tian	16-03-09	Wrtitten For NetFPGA2 Card in Microsoft Research Asia
Notes:

--*/

#ifndef _NIC_DEF_H
#define _NIC_DEF_H

// packet and header sizes
#define NIC_MAX_PACKET_SIZE		1514
#define NIC_MIN_PACKET_SIZE		60
#define NIC_HEADER_SIZE			14

// multicast list size
#define NIC_MAX_MCAST_LIST			32

// local data buffer size (to copy send packet data into a local buffer)
#define NIC_BUFFER_SIZE					1520

// max lookahead size
#define NIC_MAX_LOOKAHEAD				(NIC_MAX_PACKET_SIZE - NIC_HEADER_SIZE)

// max number of send packets the MiniportSendPackets function can accept
#define NIC_MAX_SEND_PACKETS			10


// delay used for link detection to minimize the init time
// change this value to match your hardware
#define NIC_LINK_DETECTION_DELAY		((LONGLONG) -MILLISECONDS_TO_100NS * 100) // 100ms
#define NIC_CHECK_FOR_HANG_DELAY		((LONGLONG) -MILLISECONDS_TO_100NS * 400) // 400ms

typedef enum _MEDIA_STATE {
    Connected = 0,
    Disconnected
} MEDIA_STATE;

#define ALIGN_16                   16

//
// The driver should put the data(after Ethernet header) at 8-bytes boundary
//
#define ETH_DATA_ALIGN                      8   // the data(after Ethernet header) should be 8-byte aligned
//
// Shift HW_RFD 0xA bytes to make Tcp data 8-byte aligned
// Since the ethernet header is 14 bytes long. If a packet is at 0xA bytes
// offset, its data(ethernet user data) will be at 8 byte boundary
//
#define HWRFD_SHIFT_OFFSET                0xA   // Shift HW_RFD 0xA bytes to make Tcp data 8-byte aligned

//
// The driver has to allocate more data then HW_RFD needs to allow shifting data
//
#define MORE_DATA_FOR_ALIGN         (ETH_DATA_ALIGN + HWRFD_SHIFT_OFFSET)
//
// Get a 8-bytes aligned memory address from a given the memory address.
// If the given address is not 8-bytes aligned, return  the closest bigger memory address
// which is 8-bytes aligned.
//
#define DATA_ALIGN(_Va)             ((PVOID)(((ULONG_PTR)(_Va) + (ETH_DATA_ALIGN - 1)) & ~(ETH_DATA_ALIGN - 1)))
//
// Get the number of bytes the final address shift from the original address
//
#define BYTES_SHIFT(_NewVa, _OrigVa) ((PUCHAR)(_NewVa) - (PUCHAR)(_OrigVa))

#define ETH_IS_LOCALLY_ADMINISTERED(Address) \
    (BOOLEAN)(((PUCHAR)(Address))[0] & ((UCHAR)0x02))


//--------------------------------------
// Some utility macros
//--------------------------------------
#ifndef min
#define min(_a, _b)     (((_a) < (_b)) ? (_a) : (_b))
#endif

#ifndef max
#define max(_a, _b)     (((_a) > (_b)) ? (_a) : (_b))
#endif

#define GetListHeadEntry(ListHead)  ((ListHead)->Flink)
#define GetListTailEntry(ListHead)  ((ListHead)->Blink)
#define GetListFLink(ListEntry)     ((ListEntry)->Flink)

#define IsSListEmpty(ListHead)  (((PSINGLE_LIST_ENTRY)ListHead)->Next == NULL)

NTSTATUS
NICGetStatsCounters(
    IN  PDEVICE_EXTENSION  pDevExt,
    IN  NDIS_OID     Oid,
    OUT PULONG64     pCounter
    );

#endif


