/*++

Copyright (c) 2009	Microsoft Corporation

Module Name:
	Nic_req.c

Abstract:
	This module handle NDIS OID ioctls. This module is not required
	if the upper edge is not NDIS.

Revision History:
	Who 		When		What						
	--------	--------	-----------------------------------------------
	Chen Tian	16-03-09	Wrtitten For NetFPGA2 Card in Microsoft Research Asia
Notes:

--*/

#include "..\inc\precomp.h"
#include "..\inc\nic_req.h"
#include "..\inc\Device.h"
#include "..\inc\reg_defines.h"
#include "..\inc\dbg.h"
#include "nic_def.h"

PCHAR DbgGetOidName(ULONG oid);

ULONG
NICHandleQueryOidRequest(
						 IN WDFQUEUE				Queue,
						 IN WDFREQUEST				Request,
						 IN PDEVICE_EXTENSION		pDevExt,
						 IN PVOID					pOutputBuffer,
						 IN SIZE_T					OutputBufferSize,
						 WDF_REQUEST_PARAMETERS 	*Params
						 )

/*++

Routine Description:

	Query an arbitrary OID value from the miniport.

Arguments:

	Queue - Default queue handle
	Request - IOCTL request handle
	Params - pointer to params structure for the request. This is
			 equivalent to the IRP stack location pointer.

Return Value:


--*/
{
	NTSTATUS					status = STATUS_SUCCESS;
	PNDISPROT_QUERY_OID		pQuery = NULL;
	NDIS_OID					Oid = 0;
	ULONG						ulInfo = 0;
	ULONG						ulInfo1 = 0;
	ULONG64 					ul64Info = 0;
	PVOID						pInfo = (PVOID) &ulInfo;
	ULONG						ulInfoLen = sizeof(ulInfo);
	PVOID						InformationBuffer = NULL;
	ULONG						InformationOutputBufferSize = 0;
	MEDIA_STATE 				CurrMediaState=0;
	NDIS_PNP_CAPABILITIES		Power_Management_Capabilities;
	UCHAR *						pChar;


	UNREFERENCED_PARAMETER( Params );

	DBGPRINT(MP_INFO, ("--> NICHandleQueryOidRequest \n"));

	do {

		pQuery = (PNDISPROT_QUERY_OID)pOutputBuffer;
		
		Oid = pQuery->Oid;
		
		if(OID_GEN_LINK_SPEED != Oid) 
		{ // To avoid flood of trace messages
			DBGPRINT(MP_INFO, ("\t%s\n",	DbgGetOidName(Oid)));
		}

		InformationBuffer = &pQuery->Data[0];
		
		InformationOutputBufferSize = (ULONG)OutputBufferSize -
							FIELD_OFFSET(NDISPROT_QUERY_OID, 
							Data
							);

		switch(Oid)
		{

		case OID_GEN_LINK_SPEED:
		case OID_GEN_MEDIA_CONNECT_STATUS:
			// since the link speed is fixed, we don't need to pass it to this driver in the Ndis driver, so it's obsolete

			DBGPRINT(MP_LOUD, ("Gen link speed | media connect status \n"));
			
			if (InformationOutputBufferSize < sizeof(ULONG))
			{
				status = STATUS_BUFFER_TOO_SMALL;
				break;
			 }

			 //WdfSpinLockAcquire(pDevExt->Lock);
			if (Oid == OID_GEN_LINK_SPEED)
			{
				ulInfo = 10000000;
			} 
			else
			{
				ulInfo = CurrMediaState;
			}
			
			break;

		case OID_802_3_PERMANENT_ADDRESS:

			DBGPRINT(MP_LOUD, ("Permanent Address \n"));
			
			if (InformationOutputBufferSize < ETH_LENGTH_OF_ADDRESS)
			{
				status = STATUS_BUFFER_TOO_SMALL;
				break;
			}

			pInfo = pDevExt->PermanentAddress;
			ulInfoLen = ETH_LENGTH_OF_ADDRESS;
			break;

		case OID_802_3_CURRENT_ADDRESS:

			DBGPRINT(MP_LOUD, ("Current Address \n"));

			if (InformationOutputBufferSize < ETH_LENGTH_OF_ADDRESS)
			{
				status = STATUS_BUFFER_TOO_SMALL;
				break;
			}

			pInfo = pDevExt->CurrentAddress;
			ulInfoLen = ETH_LENGTH_OF_ADDRESS;
			break;
		case OID_802_3_MAXIMUM_LIST_SIZE:

			DBGPRINT(MP_LOUD, ("Maximum list size \n"));

			if (InformationOutputBufferSize < sizeof(ULONG))
			{
				status = STATUS_BUFFER_TOO_SMALL;
				break;
			}

			ulInfo = NIC_MAX_MCAST_LIST;
			break;

		case OID_GEN_XMIT_OK:
		case OID_GEN_RCV_OK:
		case OID_GEN_RCV_ERROR:
		case OID_GEN_RCV_NO_BUFFER:
		case OID_GEN_RCV_CRC_ERROR:
			if (InformationOutputBufferSize < sizeof(ULONG))
			{
				status = STATUS_BUFFER_TOO_SMALL;
				break;
			}

			ulInfoLen = sizeof(ul64Info);
			status = NICGetStatsCounters(pDevExt, Oid, &ul64Info);
			if (status == STATUS_SUCCESS)
			{
				ulInfoLen = min(InformationOutputBufferSize, ulInfoLen);
				pInfo = &ul64Info;
			}
			break;

		/* handle the BCube requests */
		case OID_GEN_GET_MAC_0:
		case OID_GEN_GET_MAC_1:
		case OID_GEN_GET_MAC_2:
		case OID_GEN_GET_MAC_3:
			if ( InformationOutputBufferSize == sizeof(UCHAR)*6 ) 
			{
				if ( Oid == OID_GEN_GET_MAC_0) 
				{
					ulInfo =  ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_MAC_0_HI_REG );
					ulInfo1 = ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_MAC_0_LO_REG );
				}
				else if ( Oid == OID_GEN_GET_MAC_1) 
				{
					ulInfo =  ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_MAC_1_HI_REG );
					ulInfo1 = ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_MAC_1_LO_REG);
				}
				else if ( Oid == OID_GEN_GET_MAC_2) 
				{
					ulInfo =  ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_MAC_2_HI_REG );
					ulInfo1 = ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_MAC_2_LO_REG );
				}
				else if ( Oid == OID_GEN_GET_MAC_3) 
				{
					ulInfo =  ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_MAC_3_HI_REG );
					ulInfo1 = ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_MAC_3_LO_REG );
				}

				pChar = (UCHAR *) InformationBuffer;
				pChar[0] = (UCHAR) ((ulInfo&0x0000ff00)>>8);
				pChar[1] = (UCHAR) ulInfo&0x000000ff;
				pChar[2] = (UCHAR) ((ulInfo1&0xff000000)>>24);
				pChar[3] = (UCHAR) ((ulInfo1&0x00ff0000)>>16);
				pChar[4] = (UCHAR) ((ulInfo1&0x0000ff00)>>8);
				pChar[5] = (UCHAR) (ulInfo1&0x000000ff);
				ulInfoLen = sizeof(UCHAR)*6;

				DBGPRINT(MP_WARN, ("ulInfo = %x, ulInfo1=%x", ulInfo, ulInfo1));
				DBGPRINT(MP_WARN, ("Get MAC Address = %02x-%02x-%02x-%02x-%02x-%02x\n",
									pChar[0],
									pChar[1],
									pChar[2],
									pChar[3],
									pChar[4],
									pChar[5])
									);
			status = STATUS_UNSUCCESSFUL;
				
			}
			break;

		default:
			DBGPRINT(MP_INFO, ("default \n"));

			status = STATUS_NOT_SUPPORTED;
			break;
		}

	} while (FALSE);

	if (status == STATUS_SUCCESS)
	{
		RtlMoveMemory(InformationBuffer, pInfo, ulInfoLen);
	}
	//
	// Adjust the size to include the structure.
	//
	ulInfoLen += FIELD_OFFSET(NDISPROT_QUERY_OID, Data);

	DBGPRINT(MP_LOUD,
				("<--NICHandleQueryOidRequest: Status %x\n",
				status));

	return ulInfoLen;

}

VOID
NICHandleSetOidRequest(
	IN WDFQUEUE 			 	Queue,
	IN WDFREQUEST			 	Request,
	IN PDEVICE_EXTENSION		pDevExt,
	IN PVOID				   		pInputBuffer,
	IN SIZE_T					InputBufferSize,
	WDF_REQUEST_PARAMETERS	*Params
	)
/*++

Routine Description:

   This routine is called to handle set OID request sent in the ioctl buffer.
   If the device is busy, we will forward the request into a queue and complete
   it later in the DPC.

Arguments:

	Queue - Default queue handle
	Request - IOCTL request handle
	Params - pointer to params structure for the request. This is
			 equivalent to the IRP stack location pointer.

Return Value:

	VOID

--*/
{
	NTSTATUS					status = STATUS_SUCCESS;
	PNDISPROT_SET_OID			pSet;
	NDIS_OID					Oid;
	ULONG						PacketFilter;
	PVOID						InformationBuffer = NULL;
	ULONG						InformationOutputBufferSize = 0;
	PVOID						pOutputBuffer;
	size_t						OutputBufferSize;
	ULONG						unUsed;
	WDF_POWER_DEVICE_STATE	newDeviceState;
	WDF_POWER_DEVICE_STATE	oldDeviceState;
	PFORWARD_TABLE				pForwardTable;
	UCHAR *						pChar;

	UNREFERENCED_PARAMETER( Params );

	DBGPRINT(MP_WARN,  ("--> NICHandleSetOidRequest\n"));

	//
	// Since the IOCTL is buffered, WdfRequestRetrieveOutputBuffer &
	// WdfRequestRetrieveInputBuffer return the same buffer pointer.
	// So make sure you read all the information you need from
	// the buffer before you write to it.
	//
	pOutputBuffer = pInputBuffer;
	OutputBufferSize = InputBufferSize;

	Oid = 0;

	do {

		if (OutputBufferSize < sizeof(NDISPROT_SET_OID))
		{
			status = STATUS_BUFFER_OVERFLOW;
			break;
		}

		pSet = (PNDISPROT_SET_OID)pOutputBuffer;

		Oid = pSet->Oid;

		InformationBuffer = &pSet->Data[0];

		InformationOutputBufferSize = (ULONG)OutputBufferSize - FIELD_OFFSET(NDISPROT_SET_OID, Data);

		DBGPRINT(MP_INFO, ("\t%s\n", DbgGetOidName(Oid)));

		switch(Oid)
		{
		case OID_GEN_ADD_FORWARD_TABLE:
			if ( InformationOutputBufferSize == sizeof(FORWARD_TABLE) ) 
			{
				pForwardTable = ( PFORWARD_TABLE) InformationBuffer;

				DBGPRINT(MP_WARN, ("Get MAC Address = %02x-%02x-%02x-%02x-%02x-%02x\n",
									pForwardTable->mac[0],
									pForwardTable->mac[1],
									pForwardTable->mac[2],
									pForwardTable->mac[3],
									pForwardTable->mac[4],
									pForwardTable->mac[5]));

				// write the information to the hardware
				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_MAC_HI_REG, (pForwardTable->mac[0]<<8) | (pForwardTable->mac[1]) );
				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_MAC_LO_REG, 
					( (pForwardTable->mac[2]<<24) | (pForwardTable->mac[3]<<16) | (pForwardTable->mac[4]<<8) | pForwardTable->mac[5] ));

				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_NEXT_HOP_IP_REG, (ULONG)pForwardTable->nha);
				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_LUT_WR_ADDR_REG, (ULONG)pForwardTable->index);

				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_LUT_RD_ADDR_REG, (ULONG)pForwardTable->index);
				DBGPRINT(MP_WARN, ("ROUTER_OP_LUT_ARP_MAC_HI_REG 0x%x",	ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_MAC_HI_REG)));
				DBGPRINT(MP_WARN, ("ROUTER_OP_LUT_ARP_MAC_LO_REG 0x%x",	ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_MAC_LO_REG)));
				DBGPRINT(MP_WARN, ("ROUTER_OP_LUT_ARP_NEXT_HOP_IP_REG 0x%x",	ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_NEXT_HOP_IP_REG)));
	
			}
			break;

		case OID_GEN_DEL_FORWARD_TABLE:
			DBGPRINT(MP_WARN, ("OID_GEN_DEL_FORWARD_TABLE"));

			if ( InformationOutputBufferSize == sizeof(UINT) ) 
			{

				// write the information to the hardware
				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_MAC_HI_REG, 0xffffffff );
				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_MAC_LO_REG, 0xffffffff);
				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_NEXT_HOP_IP_REG, 0xffffffff);
				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_LUT_WR_ADDR_REG, (ULONG)*(PUINT) InformationBuffer);

				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_LUT_RD_ADDR_REG, (ULONG)*(PUINT) InformationBuffer);
				DBGPRINT(MP_WARN, ("ROUTER_OP_LUT_ARP_MAC_HI_REG 0x%x",	ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_MAC_HI_REG)));
				DBGPRINT(MP_WARN, ("ROUTER_OP_LUT_ARP_MAC_LO_REG 0x%x",	ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_MAC_LO_REG)));
				DBGPRINT(MP_WARN, ("ROUTER_OP_LUT_ARP_NEXT_HOP_IP_REG 0x%x",	ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_ARP_NEXT_HOP_IP_REG)));
			}
			break;

		case OID_GEN_BCN_ADDRESS:
			if ( InformationOutputBufferSize == sizeof(UINT) ) 
			{
				
				// write the information to the hardware
				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_DST_IP_FILTER_IP_REG, (ULONG) *(PUINT) InformationBuffer );
				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_DST_IP_FILTER_WR_ADDR_REG, 0);
				WriteRegisterUlong (pDevExt, ROUTER_OP_LUT_DST_IP_FILTER_RD_ADDR_REG, 0);
				DBGPRINT(MP_WARN, ("ROUTER_OP_LUT_ARP_NEXT_HOP_IP_REG 0x%x",	ReadRegisterUlong (pDevExt, ROUTER_OP_LUT_DST_IP_FILTER_IP_REG)));
			}
			break;

		default:
			status = STATUS_NOT_SUPPORTED;
			break;
		}
	} while (FALSE);

	DBGPRINT(MP_INFO,  ("<-- NICHandleSetOidRequest %x\n", status));

	return;
}

NTSTATUS
NICGetStatsCounters(
	IN	PDEVICE_EXTENSION  	pDevExt,
	IN	NDIS_OID	 		Oid,
	OUT PULONG64	 		pCounter
	)
/*++
Routine Description:

	Get the value for a statistics OID

Arguments:

	FdoData 	Pointer to our FdoData
	Oid 		Self-explanatory
	pCounter	Pointer to receive the value

Return Value:

	NT Status code

--*/
{
	NTSTATUS	 status = STATUS_SUCCESS;

	DBGPRINT ( MP_INFO, ("---> NICGetStatsCounters"));

	*pCounter = 0;

	switch(Oid)
	{
		case OID_GEN_XMIT_OK:
			*pCounter = ReadRegisterUlong (pDevExt, TX_QUEUE_0_NUM_PKTS_SENT_REG) +
				ReadRegisterUlong (pDevExt, TX_QUEUE_1_NUM_PKTS_SENT_REG) +
				ReadRegisterUlong (pDevExt, TX_QUEUE_2_NUM_PKTS_SENT_REG) +
				ReadRegisterUlong (pDevExt, TX_QUEUE_3_NUM_PKTS_SENT_REG);
			break;

		case OID_GEN_RCV_OK:
			*pCounter = ReadRegisterUlong (pDevExt, RX_QUEUE_0_NUM_PKTS_STORED_REG) +
				ReadRegisterUlong (pDevExt, RX_QUEUE_1_NUM_PKTS_STORED_REG) +
				ReadRegisterUlong (pDevExt, RX_QUEUE_2_NUM_PKTS_STORED_REG) +
				ReadRegisterUlong (pDevExt, RX_QUEUE_3_NUM_PKTS_STORED_REG);
			break;

		case OID_GEN_XMIT_ERROR:
			*pCounter = 0;
			break;

		case OID_GEN_RCV_ERROR:
			*pCounter = ReadRegisterUlong (pDevExt, RX_QUEUE_0_NUM_PKTS_DROPPED_FULL_REG) +
						ReadRegisterUlong (pDevExt, RX_QUEUE_0_NUM_PKTS_DROPPED_BAD_REG );
			break;

		case OID_GEN_RCV_NO_BUFFER:
			*pCounter = ReadRegisterUlong (pDevExt, RX_QUEUE_0_NUM_PKTS_DROPPED_FULL_REG);
			break;

		case OID_GEN_RCV_CRC_ERROR:
			*pCounter = ReadRegisterUlong (pDevExt, RX_QUEUE_0_NUM_PKTS_DROPPED_BAD_REG );
			break;

		case OID_GEN_TRANSMIT_QUEUE_LENGTH:
			*pCounter = 0;
			break;

		case OID_802_3_RCV_ERROR_ALIGNMENT:
			*pCounter = 0;
			break;

		case OID_802_3_XMIT_ONE_COLLISION:
			*pCounter = 0;
			break;

		case OID_802_3_XMIT_MORE_COLLISIONS:
			*pCounter = 0;
			break;

		case OID_802_3_XMIT_DEFERRED:
			*pCounter = 0;
			break;

		case OID_802_3_XMIT_MAX_COLLISIONS:
			*pCounter = 0;
			break;

		case OID_802_3_RCV_OVERRUN:
			*pCounter = 0;
			break;

		case OID_802_3_XMIT_UNDERRUN:
			*pCounter = 0;
			break;

		case OID_802_3_XMIT_HEARTBEAT_FAILURE:
			*pCounter = 0;
			break;

		case OID_802_3_XMIT_TIMES_CRS_LOST:
			*pCounter = 0;
			break;

		case OID_802_3_XMIT_LATE_COLLISIONS:
			*pCounter = 0;
			break;

		default:
			status = STATUS_NOT_SUPPORTED;
			break;
	}

	DBGPRINT ( MP_INFO, ("<--- NICGetStatsCounters"));

	return(status);
}

PCHAR
DbgGetOidName(ULONG oid)
{
	PCHAR oidName;

	switch (oid){

		#undef MAKECASE
		#define MAKECASE(oidx) case oidx: oidName = #oidx; break;

		MAKECASE(OID_GEN_SUPPORTED_LIST)
		MAKECASE(OID_GEN_HARDWARE_STATUS)
		MAKECASE(OID_GEN_MEDIA_SUPPORTED)
		MAKECASE(OID_GEN_MEDIA_IN_USE)
		MAKECASE(OID_GEN_MAXIMUM_LOOKAHEAD)
		MAKECASE(OID_GEN_MAXIMUM_FRAME_SIZE)
		MAKECASE(OID_GEN_LINK_SPEED)
		MAKECASE(OID_GEN_TRANSMIT_BUFFER_SPACE)
		MAKECASE(OID_GEN_RECEIVE_BUFFER_SPACE)
		MAKECASE(OID_GEN_TRANSMIT_BLOCK_SIZE)
		MAKECASE(OID_GEN_RECEIVE_BLOCK_SIZE)
		MAKECASE(OID_GEN_VENDOR_ID)
		MAKECASE(OID_GEN_VENDOR_DESCRIPTION)
		MAKECASE(OID_GEN_CURRENT_PACKET_FILTER)
		MAKECASE(OID_GEN_CURRENT_LOOKAHEAD)
		MAKECASE(OID_GEN_DRIVER_VERSION)
		MAKECASE(OID_GEN_MAXIMUM_TOTAL_SIZE)
		MAKECASE(OID_GEN_PROTOCOL_OPTIONS)
		MAKECASE(OID_GEN_MAC_OPTIONS)
		MAKECASE(OID_GEN_MEDIA_CONNECT_STATUS)
		MAKECASE(OID_GEN_MAXIMUM_SEND_PACKETS)
		MAKECASE(OID_GEN_VENDOR_DRIVER_VERSION)
		MAKECASE(OID_GEN_SUPPORTED_GUIDS)
		MAKECASE(OID_GEN_NETWORK_LAYER_ADDRESSES)
		MAKECASE(OID_GEN_TRANSPORT_HEADER_OFFSET)
		MAKECASE(OID_GEN_MEDIA_CAPABILITIES)
		MAKECASE(OID_GEN_PHYSICAL_MEDIUM)
		MAKECASE(OID_GEN_XMIT_OK)
		MAKECASE(OID_GEN_RCV_OK)
		MAKECASE(OID_GEN_XMIT_ERROR)
		MAKECASE(OID_GEN_RCV_ERROR)
		MAKECASE(OID_GEN_RCV_NO_BUFFER)
		MAKECASE(OID_GEN_DIRECTED_BYTES_XMIT)
		MAKECASE(OID_GEN_DIRECTED_FRAMES_XMIT)
		MAKECASE(OID_GEN_MULTICAST_BYTES_XMIT)
		MAKECASE(OID_GEN_MULTICAST_FRAMES_XMIT)
		MAKECASE(OID_GEN_BROADCAST_BYTES_XMIT)
		MAKECASE(OID_GEN_BROADCAST_FRAMES_XMIT)
		MAKECASE(OID_GEN_DIRECTED_BYTES_RCV)
		MAKECASE(OID_GEN_DIRECTED_FRAMES_RCV)
		MAKECASE(OID_GEN_MULTICAST_BYTES_RCV)
		MAKECASE(OID_GEN_MULTICAST_FRAMES_RCV)
		MAKECASE(OID_GEN_BROADCAST_BYTES_RCV)
		MAKECASE(OID_GEN_BROADCAST_FRAMES_RCV)
		MAKECASE(OID_GEN_RCV_CRC_ERROR)
		MAKECASE(OID_GEN_TRANSMIT_QUEUE_LENGTH)
		MAKECASE(OID_GEN_GET_TIME_CAPS)
		MAKECASE(OID_GEN_GET_NETCARD_TIME)
		MAKECASE(OID_GEN_NETCARD_LOAD)
		MAKECASE(OID_GEN_DEVICE_PROFILE)
		MAKECASE(OID_GEN_INIT_TIME_MS)
		MAKECASE(OID_GEN_RESET_COUNTS)
		MAKECASE(OID_GEN_MEDIA_SENSE_COUNTS)
		MAKECASE(OID_PNP_CAPABILITIES)
		MAKECASE(OID_PNP_SET_POWER)
		MAKECASE(OID_PNP_QUERY_POWER)
		MAKECASE(OID_PNP_ADD_WAKE_UP_PATTERN)
		MAKECASE(OID_PNP_REMOVE_WAKE_UP_PATTERN)
		MAKECASE(OID_PNP_ENABLE_WAKE_UP)
		MAKECASE(OID_802_3_PERMANENT_ADDRESS)
		MAKECASE(OID_802_3_CURRENT_ADDRESS)
		MAKECASE(OID_802_3_MULTICAST_LIST)
		MAKECASE(OID_802_3_MAXIMUM_LIST_SIZE)
		MAKECASE(OID_802_3_MAC_OPTIONS)
		MAKECASE(OID_802_3_RCV_ERROR_ALIGNMENT)
		MAKECASE(OID_802_3_XMIT_ONE_COLLISION)
		MAKECASE(OID_802_3_XMIT_MORE_COLLISIONS)
		MAKECASE(OID_802_3_XMIT_DEFERRED)
		MAKECASE(OID_802_3_XMIT_MAX_COLLISIONS)
		MAKECASE(OID_802_3_RCV_OVERRUN)
		MAKECASE(OID_802_3_XMIT_UNDERRUN)
		MAKECASE(OID_802_3_XMIT_HEARTBEAT_FAILURE)
		MAKECASE(OID_802_3_XMIT_TIMES_CRS_LOST)
		MAKECASE(OID_802_3_XMIT_LATE_COLLISIONS)

		default:
			oidName = "<** UNKNOWN OID **>";
			break;
	}

	return oidName;
}


